ShareLobbyLink = ShareLobbyLink or {}
ShareLobbyLink._mod_path = ShareLobbyLink._mod_path or ModPath
ShareLobbyLink._default_localization_path = ShareLobbyLink._default_localization_path or (ShareLobbyLink._mod_path .. "loc/en.json")
ShareLobbyLink._menu_path = ShareLobbyLink._menu_path or (ShareLobbyLink._mod_path .. "menu/options.json")
ShareLobbyLink._save_path = ShareLobbyLink._save_path or (SavePath .. "sharelobbylink.json")
ShareLobbyLink._delayed_calls_timeout_id = "ShareLobbyLink_JoinLobby"
ShareLobbyLink._game_id = "218620"
ShareLobbyLink._join_link_protocol_prefix = "steam://joinlobby/"
ShareLobbyLink.settings = {
	timeout_duration = 10,
	log_enabled = false,
	skip_dialog = false,
	invite_button_visible = true,
	join_button_visible = true,
	text_color_mouseover = "ffffff",
	text_color_default = "007aff",
	bg_color = "122d46",
	animate_click_duration = 0.25
}
ShareLobbyLink._is_joining_lobby = false

function ShareLobbyLink.table_concat(tbl,div)
	div = tostring(div or ",")
	if type(tbl) ~= "table" then 
		return "(concat error: non-table value)"
	end
	local str
	for k,v in ipairs(tbl) do
		str = str and (str .. div .. tostring(v)) or tostring(v)
	end
	return str or ""
end

function ShareLobbyLink.animate_clicked(o,duration,color_1,color_2)
	duration = duration or ShareLobbyLink.settings.animate_click_duration or 1
	
	local t = 0
	while t < duration do
		t = t + coroutine.yield()
		local color_1 = color_1 or Color(ShareLobbyLink.settings.text_color_default)
		local color_2 = color_2 or Color(ShareLobbyLink.settings.text_color_mouseover)
		local n = 1 - ((duration - t) / duration)
		local r = math.lerp(color_1.r, color_2.r, n)
		local g = math.lerp(color_1.g, color_2.g, n)
		local b = math.lerp(color_1.b, color_2.b, n)

		o:set_color(Color(r, g, b))
	end
end


function ShareLobbyLink:log(...)
	if self.settings.log_enabled then 
		log("[ShareLobbyLink]: " .. self.table_concat({...},", "))
	end
end

function ShareLobbyLink:callback_dialog_invalid_lobby(clipboard,lobby_id)
	if self.settings.skip_dialog then 
		return
	end
	
	local title = managers.localization:text("menu_sll_dialogue_invalid_lobby_title")
	local desc = string.gsub(managers.localization:text("menu_sll_dialogue_invalid_lobby_desc"),"$S1",tostring(clipboard))
	desc = string.gsub(desc,"$S2",tostring(lobby_id))
	QuickMenu:new(title,desc,{
		{
			text = managers.localization:text("menu_back"),
			is_cancel_button = true
		}
	},true)
end

function ShareLobbyLink:callback_dialog_invalid_link(clipboard)
	if self.settings.skip_dialog then 
		return
	end
	
	local title = managers.localization:text("menu_sll_dialogue_invalid_link_title")
	local desc = string.gsub(managers.localization:text("menu_sll_dialogue_invalid_link_desc"),"$S1",tostring(clipboard))
	QuickMenu:new(title,desc,{
		{
			text = managers.localization:text("menu_back"),
			is_cancel_button = true
		}
	},true)
end

function ShareLobbyLink:callback_dialog_no_lobby()
	if self.settings.skip_dialog then 
		return
	end
	
	local title = managers.localization:text("menu_sll_dialogue_not_in_lobby_title")
	local desc = string.gsub(managers.localization:text("menu_sll_dialogue_not_in_lobby_desc"),"$S1",managers.localization:text("menu_crimenet"))
	QuickMenu:new(title,desc,{
		{
			text = managers.localization:text("menu_back"),
			is_cancel_button = true
		}
	},true)
end

function ShareLobbyLink:callback_dialog_request_timeout(lobby_id,time_waited)
	if self.settings.skip_dialog then 
		return
	end
	
	local title = managers.localization:text("menu_sll_dialogue_request_timeout_title")
	local desc = string.gsub(managers.localization:text("menu_sll_dialogue_request_timeout_desc"),"$S1",tostring(lobby_id))
	desc = string.format(desc,time_waited)
	QuickMenu:new(title,desc,{
		{
			text = managers.localization:text("menu_back"),
			is_cancel_button = true
		}
	},true)
end

function ShareLobbyLink:callback_dialog_copy_success()
	if self.settings.skip_dialog then 
		return
	end
	
	local title = managers.localization:text("menu_sll_dialogue_copy_success_title")
	local desc = managers.localization:text("menu_sll_dialogue_copy_success_desc")
	QuickMenu:new(title,desc,{
		{
			text = managers.localization:text("menu_back"),
			is_cancel_button = true
		}
	},true)
end

function ShareLobbyLink:join_lobby_from_clipboard()
	managers.system_menu:close("join_server")
	
	local delayedcalls_timeout_id = self._delayed_calls_timeout_id
	local delayed_calls = DelayedCallsFix or DelayedCalls
	local timeout_duration = self.settings.timeout_duration
--	delayed_calls:Remove(delayedcalls_timeout_id)
	-- unfortunately, DelayedCalls queues removals but does not preserve the execution order, so Remove() and then Add() will remove the new delayed call on the same frame.
	
	self._is_joining_lobby = false
	local _clipboard = Application:get_clipboard()
	local lobby_id
	
	local matchmake_manager = managers.network.matchmake
	if not matchmake_manager then
		self:log("join_lobby_from_clipboard() failed- NetworkMatchMakingSTEAM not initialized!")
		self:callback_dialog_no_lobby()
		return
	end
	
--	if is_in_game then
		--!
--	end
	
	if _clipboard and _clipboard ~= "" then
		local clipboard = tostring(_clipboard)
		local a,b = string.find(clipboard,self._join_link_protocol_prefix)
		if b then 
			local s1 = string.sub(clipboard,b)
			local split = string.split(s1,"/")
			lobby_id = split and split[2] or s1
			
			local join_target_id64 = split[3]
		else
			lobby_id = string.match(clipboard,"%d+")
		end
	end
	
	local start_t = Application:time()
	
	local lobby = lobby_id and Steam:lobby(lobby_id)
	if lobby then 
	
		if matchmake_manager.lobby_handler then 
			if lobby_id == matchmake_manager.lobby_handler:id() then 
				self:log("join_lobby_from_clipboard() failed- you are already in this lobby!")
				return
			end
		end

		local cb_request_data = function()
			managers.system_menu:close("join_server")
			lobby:setup_callback(function() end)
			delayed_calls:Remove(delayedcalls_timeout_id)
			local owner_id = lobby:key_value("owner_id")
			if owner_id and self._is_joining_lobby then 
				self:log("Joining lobby... [lobby_id: " .. tostring(lobby_id) .. " ] [owner name: " .. tostring(lobby:key_value("owner_name")) ..  "] [name: " .. tostring(lobby:key_value("owner_name")) .. "]")
--				matchmake_manager:join_server_with_check(lobby_id,true)
				NetworkAccountSTEAM._on_join_request(lobby_id,nil) --this function leaves the current game before joining the new game, mitigating a crash
			else
				self:log(owner_id,self._is_joining_lobby)
			end
			self._is_joining_lobby = false
		end
		
		self._is_joining_lobby = true
		managers.menu:show_joining_lobby_dialog()
		
		delayed_calls:Add(delayedcalls_timeout_id,timeout_duration,function()
			managers.system_menu:close("join_server")
			if self._is_joining_lobby then 
				self:callback_dialog_request_timeout(lobby_id,Application:time() - start_t)
				self._is_joining_lobby = false
			end
		end)
		
		lobby:setup_callback(cb_request_data)
		if lobby:key_value("state") == "value_pending" then
			self:log("Requesting lobby data from: " .. tostring(lobby_id)) 
			lobby:request_data()
		else
			self:log("Lobby data already loaded.")
			cb_request_data()
		end
	else
		self:log("join_lobby_from_clipboard() failed - no join link found on clipboard: " .. tostring(_clipboard))
		self:callback_dialog_invalid_link(_clipboard)
		return
	end
end

function ShareLobbyLink:copy_lobby_link_to_clipboard()
	
	local lobby_handler = managers.network.matchmake and managers.network.matchmake.lobby_handler
	if not lobby_handler then
		self:log("copy_lobby_link_to_clipboard() failed- NetworkMatchMakingSTEAM not initialized!")
		self:callback_dialog_no_lobby()
		return
	end
	
	local _,host_id,owner = lobby_handler:get_server_details()
	
	local lobby_id = lobby_handler:id()
	
	if not lobby_id then 
		self:log("copy_lobby_link_to_clipboard() failed- you are not currently in a lobby!")
		self:callback_dialog_no_lobby()
		return
	end
	
	local game_id = self._game_id
	local join_link = self._join_link_protocol_prefix .. game_id .. "/" .. tostring(lobby_id) .. "/" .. tostring(host_id)
	
	Application:set_clipboard(join_link)
	self:log("Copied link to clipboard: " .. tostring(join_link))
	managers.system_menu:close("join_server")
	self:callback_dialog_copy_success()
end


function ShareLobbyLink:Load()
	local file = io.open(ShareLobbyLink._save_path, "r")
	if (file) then
		for k, v in pairs(json.decode(file:read("*all"))) do
			ShareLobbyLink.settings[k] = v
		end
	else
		ShareLobbyLink:Save()
	end
end

function ShareLobbyLink:Save()
	local file = io.open(ShareLobbyLink._save_path,"w+")
	if file then
		file:write(json.encode(ShareLobbyLink.settings))
		file:close()
	end
end


Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_SLL", function( loc )
	if not BeardLib then
		loc:load_localization_file(ShareLobbyLink._default_localization_path)
	end
end)

Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_SLL", function(menu_manager)

	MenuCallbackHandler.callback_sll_join_button_visible = function(self,item)
		ShareLobbyLink.settings.join_button_visible = item:value() == "on"
		ShareLobbyLink:Save()
	end
	MenuCallbackHandler.callback_sll_invite_button_visible = function(self,item)
		ShareLobbyLink.settings.invite_button_visible = item:value() == "on"
		ShareLobbyLink:Save()
	end
	MenuCallbackHandler.callback_sll_log_enabled = function(self,item)
		ShareLobbyLink.settings.log_enabled = item:value() == "on"
		ShareLobbyLink:Save()
	end
	MenuCallbackHandler.callback_sll_skip_dialog_enabled = function(self,item)
		ShareLobbyLink.settings.skip_dialog = item:value() == "on"
		ShareLobbyLink:Save()
	end
	MenuCallbackHandler.callback_sll_timeout_slider = function(self,item)
		ShareLobbyLink.settings.timeout_duration = tonumber(item:value())
		ShareLobbyLink:Save()
	end
	MenuCallbackHandler.callback_sll_copy_link = function(self)
		ShareLobbyLink:copy_lobby_link_to_clipboard()
	end
	
	MenuCallbackHandler.callback_sll_paste_link = function(self)
		if not Utils:IsInHeist() then
			ShareLobbyLink:join_lobby_from_clipboard()
		end
	end
	
	ShareLobbyLink:Load()
	MenuHelper:LoadFromJsonFile(ShareLobbyLink._menu_path,ShareLobbyLink,ShareLobbyLink.settings)
end)